/***********************
 * <lazy/exp_base.hxx> *
 ***********************/
// $Id: exp_base.hxx 196 2004-03-31 18:02:14Z raph $

/* this file is part of the Vaucanson project */
#ifndef LAZY_EXP_BASE_HXX
#define LAZY_EXP_BASE_HXX


#include <lazy/exp_base.hh>
#include <cassert>


namespace vcsn
{
  namespace vauc
  {

    namespace lazy
    {

      /*-------------------------------------.
      | Argument vector for lazy expressions |
      `-------------------------------------*/

      template<typename Exp, typename Kind>
      inline ExpArgs<Exp, Kind>::ExpArgs() 
      { 
	for (size_t i = 0; i < Kind::arity; ++i) 
	  tab_[i] = 0; 
      }

      template<typename Exp, typename Kind>
      inline Exp& ExpArgs<Exp, Kind>::operator[](typename Kind::types i) const
      { 
	assert(tab_[i] != 0);
	return *tab_[i]; 
      }

      template<typename Exp, typename Kind>
      inline ExpArgs<Exp, Kind>::~ExpArgs()
      {
	for (size_t i = 0; i < Kind::arity; ++i)
	  delete tab_[i];
      }

      template<typename Exp, typename Kind>
      inline void ExpArgs<Exp, Kind>::set(typename Kind::types i, const Exp& e)
      {
	assert(& e != 0);
	if (tab_[i] != &e)
	  delete tab_[i];
	tab_[i] = e.clone();
      }

      template<typename Exp, typename Kind>
      inline void ExpArgs<Exp, Kind>::attach(typename Kind::types i, Exp* e)
      {
	assert(e != 0);
	if (tab_[i] != e)
	  delete tab_[i];
	tab_[i] = e;
      }

      template<typename Exp, typename Kind>
      inline ExpArgs<Exp, Kind>& ExpArgs<Exp, Kind>::operator=(const ExpArgs<Exp, Kind>& other)
      {
	for (size_t i = 0; i < Kind::arity; ++i)
	  {
	    if (tab_[i] != other.tab_[i])
	      delete tab_[i];
	    tab_[i] = other.tab_[i].clone();
	  }
      }

      template<typename Exp, typename Kind>
      inline ExpArgs<Exp, Kind>::ExpArgs(const ExpArgs<Exp, Kind>& other)
      {
	for (size_t i = 0; i < Kind::arity; ++i)
	  {
	    if (other.tab_[i])
	      tab_[i] = other.tab_[i]->clone();
	    else
	      tab_[i] = 0;
	  }
      }
      

      /*----------------------------------.
      | Generic expression implementation |
      `----------------------------------*/

      template<typename Visitor, typename Kind>
      inline void ExpImpl<Visitor, Kind>::accept(Visitor& v)
      {
	v.visit(args());
      }

      template<typename Visitor, typename Kind>
      inline ExpImpl<Visitor, Kind>* ExpImpl<Visitor, Kind>::attach(typename Kind::types i, Exp<Visitor>* e)
      {
	assert(e != 0);
	args().attach(i, e);
	return this;
      }
      
      template<typename V, typename Kind>
      inline ExpArgs<Exp<V>, Kind>& ExpImpl<V, Kind>::args()
      { return args_; }

      template<typename V, typename Kind>
      inline const ExpArgs<Exp<V>, Kind>& ExpImpl<V, Kind>::args() const
      { return args_; }

      template<typename V, typename Kind>
      inline Exp<V>* ExpImpl<V, Kind>::clone() const
      { return new ExpImpl<V, Kind>(*this); }

      

    }
  }
}


#endif
